-------------------------------------------------------------------------------
--                L  O  C  A  L     V  A  R  I  A  B  L  E  S
-------------------------------------------------------------------------------

local addonName, L = ...;

local isHooked = false;

local rewardsFrames =
{
  {
    name  = "QuestInfoRewardsFrame",
    icons = {}
  },
  {
    name  = "MapQuestInfoRewardsFrame",
    icons = {}
  }
};

-------------------------------------------------------------------------------
--                L  O  C  A  L     F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

--
-- Function to hide all the valuable icons for a rewards frame
--
local function valuableIconHide(frameInfo)
  -- Hide all the valuable icons
  for _, icon in ipairs(frameInfo.icons) do
    icon:Hide();
  end
  frameInfo.icons[0] = 0;

  return;
end

--
-- Function to show the next valuable icon on a reward button
--
local function valuableIconShow(frameInfo, button)
  -- Local Variables
  local iconTexture;
  local iconFrame;
  local iconIdx;

  -- Work out which icon to use
  iconIdx   = (frameInfo.icons[0] or 0)+1;
  iconFrame = frameInfo.icons[iconIdx];

  -- Create the frame and texture for a valuable icon, if required
  if (iconFrame == nil) then
    iconFrame = CreateFrame("FRAME", nil, _G[frameInfo.name], nil);
    iconFrame:SetSize(16, 16);
    iconTexture = iconFrame:CreateTexture(nil, "BORDER");
    iconTexture:SetAtlas("bags-junkcoin", true);
    iconTexture:SetPoint("TOPLEFT", 0, 0);
    frameInfo.icons[iconIdx] = iconFrame;
  end

  -- Position and show the icon
  iconFrame:SetPoint("TOPLEFT", button, 0, 0);
  iconFrame:Show();
  frameInfo.icons[0] = iconIdx;

  return frame;
end

-------------------------------------------------------------------------------
--               G  L  O  B  A  L     F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

--
-- Function to update the Quest Item buttons to indicate which reward is most
-- valuable when sold to a merchant
--
function NigglesQuestsItemInfoUpdate()
  -- Local Variables
  local itemLink;
  local itemCount;
  local itemValue;
  local maxValueButtons = {};
  local maxValue = 0;
  local numMaxValue = 0;
  local missingInfo = false;
  local rewardButtons;
  local numShown = 0;

  -- Check the most valuable quest reward is to be indicated
  if (NigglesDBC.settings.UiQuestRewards) then
    -- Process all quest rewards frames
    for frameIdx = 1, #rewardsFrames do
      -- Hide all the valuable icons
      valuableIconHide(rewardsFrames[frameIdx]);

      -- Check at least one item is shown in the current rewards frame
      rewardFrame = _G[rewardsFrames[frameIdx].name];
      if ((rewardFrame ~= nil) and
          (rewardFrame:IsShown()) and
          (rewardFrame.RewardButtons[1] ~= nil) and
          (rewardFrame.RewardButtons[1]:IsShown())) then
        -- Assign info to more convenient variables
        rewardButtons = rewardFrame.RewardButtons;

        -- Get the item info for all buttons showing quest rewards
        for _, button in ipairs(rewardButtons) do
          -- Check the button is showing a quest reward
          if (button:IsShown()) then
            -- Check the button is showing a choice
            if ((button.type       == "choice") and
                (button.objectType == "item")) then
              -- Get the link for the button's item
              if (QuestInfoFrame.questLog) then
                itemLink = GetQuestLogItemLink("choice", button:GetID());
                _, _, itemCount = GetQuestLogChoiceInfo(button:GetID());
              else
                itemLink = GetQuestItemLink("choice", button:GetID());
                _, _, itemCount = GetQuestItemInfo("choice", button:GetID());
              end
              if ((itemLink ~= nil) and (itemCount ~= nil)) then
                -- Get the item's value
                _, _, _, _, _, _, _, _, _, _, itemValue = GetItemInfo(itemLink);
                if (itemValue ~= nil) then
                  itemValue = itemValue*itemCount;
                  if (itemValue > maxValue) then
                    maxValue    = itemValue;
                    numMaxValue = 0;
                  end
                  if (itemValue == maxValue) then
                    numMaxValue = numMaxValue+1;
                    maxValueButtons[numMaxValue] = button;
                  end
                else
                  maxValue    = 0;
                  numMaxValue = 0;
                  missingInfo = true;
                  break;
                end
              else
                maxValue    = 0;
                numMaxValue = 0;
                missingInfo = true;
                break;
              end
              numShown = numShown+1;
            end
          end
        end

        -- Show the icons for the most valuable items
        if ((maxValue > 0) and (numShown > 1) and (numMaxValue >= 1)) then
          for idx = 1, numMaxValue do
            valuableIconShow(rewardsFrames[frameIdx], maxValueButtons[idx]);
          end
        end
      end
    end

    -- Check if the value of any items couldn't be obtained
    if ((NigglesDBC.settings.UiQuestRewards) and (missingInfo)) then
      C_Timer.After(0.25, NigglesQuestsItemInfoUpdate);
    end
  end

  return;
end

--
-- Function to enable/disable the Niggles quests features
--
function NigglesQuestsEnable()
  -- Hook the function that updates the Quest Info Item buttons, if required
  if (not isHooked) then
    hooksecurefunc("QuestInfo_Display", NigglesQuestsItemInfoUpdate);
    isHooked = true;
  end

  -- Check if the quest info frames actually need to be enabled/disabled
  if (not NigglesDBC.settings.UiQuestRewards) then
    -- Hide the most valuable quest reward indicators
    for idx = 1, #rewardsFrames do
      valuableIconHide(rewardsFrames[frameIdx]);
    end
  end

  return;
end
